# bridge.py

from multiprocessing.managers import BaseManager
import threading
import socket
import traceback


class ConnectionManager():
    def __init__(self):
        self.remote_manager = BaseManager(address=('localhost', 5001),
                                          authkey=b'my_server',
                                          serializer='xmlrpclib')
        self.remote_manager.register('items_buffer')
        self.items_buffer = None
        self.items_buffer_lock = threading.Lock()
        self.connecting = False
        self.connecting_lock = threading.Lock()
        self.connection_started_condition = threading.Condition()
        
    def transmit_item(self, item):
        try:
            with self.items_buffer_lock:
                print('items_buffer memory id:', id(self.items_buffer))
                self.items_buffer.put(item)
        except (AttributeError, EOFError, IOError):
            traceback.print_exc()
            with self.connection_started_condition:
                with self.connecting_lock:
                    if not self.connecting:
                        print('trying to connect')
                        self.connecting = True
                    
                        connect_th = threading.Thread(target=self.connect_to_server,
                                                      name='Client Connect')
                        connect_th.start()
                
                self.connection_started_condition.notify()
        
            raise ConnectionError('Connection Error')
        
    def connect_to_server(self):
        with self.connection_started_condition:
            self.connection_started_condition.wait()
        
        try:
            self.remote_manager.connect()
        except socket.error:
            pass
        else:
            try:
                with self.items_buffer_lock:
                    self.items_buffer = self.remote_manager.items_buffer()
            except (AssertionError, socket.error):
                pass
            else:
                # Test to ensure it works again
                with self.items_buffer_lock:
                    self.items_buffer.put('reconnection OK')
                    print('new items_buffer memory id:', id(self.items_buffer))
            
        with self.connecting_lock:
            self.connecting = False
        
        
class ConnectionError(Exception):
    def __init__(self, value):
        self.value = value
    
    def __str__(self):
        return repr(self.value)
